
#include <time.h>
#include <SDL.h>
#include <SDL_thread.h>
#include <SDL_ttf.h>
#include <SDL_events.h>
#include <SDL_image.h>
#include <SDL_gfxPrimitives.h>
#include <SDL_rotozoom.h>
#include <SDL_opengl.h>

typedef struct {
        int x;
        int y;
        int z;
        int center_x;
        int center_y;
        int center_z;
        int rotate_x;
        int rotate_y;
        int rotate_z;
        int action_rx;
        int action_ry;
        int action_rz;
        int view;
        int group;
        char name[256];
	GLuint object;
} Gl_Element;
Gl_Element Part[100];

char obj_list[4096];
GLuint texture[100];
int gl_w = 100;
int gl_h = 100;

int glload_object(int obj_n, char *filename) {
	FILE *stream;
	int n = 0;
	int x = 0;
	int y = 0;
	int z = 0;
	int r = 0;
	int g = 0;
	int b = 0;
	int a = 0;
	Part[obj_n].x = 0;
	Part[obj_n].y = 0;
	Part[obj_n].z = -0;
	Part[obj_n].center_x = 0;
	Part[obj_n].center_y = 0;
	Part[obj_n].center_z = 0;
	Part[obj_n].rotate_x = 0;
	Part[obj_n].rotate_y = 0;
	Part[obj_n].rotate_z = 0;
	Part[obj_n].action_rx = 0;
	Part[obj_n].action_ry = 0;
	Part[obj_n].action_rz = 0;
	Part[obj_n].view = 1;
	Part[obj_n].group = 0;
	strcpy(Part[obj_n].name, "Object: export-3d.dat");
	glMatrixMode(GL_MODELVIEW);
	Part[obj_n].object = glGenLists(1);
	glNewList(Part[obj_n].object, GL_COMPILE);
		char read_line[255];
		if ((stream = fopen(filename, "r")) != 0) {
			n = 0;
			while(!feof(stream)) {
				read_line[0] = 0;
				read_line[255] = 0;
				fgets(read_line, 254, stream);
				if (strstr(read_line, "\n") != 0) {
					read_line[strstr(read_line, "\n") - read_line] = 0;
				}
				if (strncmp(read_line, "LINES;", 6) == 0) {
					glBegin( GL_LINES );
				} else if (strncmp(read_line, "QUADS;", 6) == 0) {
					glBegin( GL_QUADS );
				} else if (strncmp(read_line, "END;", 4) == 0) {
					glEnd();
				} else if (strncmp(read_line, "ORTHO ", 6) == 0) {
					sscanf(read_line + 6, "%i %i %i", &x, &y, &z);
					glMatrixMode(GL_PROJECTION);
					glLoadIdentity();
					gl_w = x;
					gl_h = y;
					glViewport(45, 50, x, y);
					glOrtho(0, (float)x, 0, (float)y, -255, 0);
					glMatrixMode(GL_MODELVIEW);
				} else if (strncmp(read_line, "COLOR ", 6) == 0) {
					sscanf(read_line + 6, "%i %i %i %i", &r, &g, &b, &a);
					glColor4f((float)r / 255, (float)g / 255, (float)b / 255, 1.0);
				} else if (strncmp(read_line, "3D ", 3) == 0) {
					sscanf(read_line + 3, "%i %i %i", &x, &y, &z);
					glVertex3f((float)x, (float)y, 255 - (float)z);
				}
			}
			fclose(stream);
		}
	glEndList();
	return 0;
}

int gldraw_object(int obj_n) {
	glPushMatrix();
	glTranslatef(Part[obj_n].center_x, Part[obj_n].center_y, Part[obj_n].center_z);
	glRotatef((float)Part[obj_n].rotate_x, 1.0, 0.0, 0.0);
	glRotatef((float)Part[obj_n].rotate_y, 0.0, 1.0, 0.0);
	glRotatef((float)Part[obj_n].rotate_z, 0.0, 0.0, 1.0);
	glTranslatef(-Part[obj_n].center_x, -Part[obj_n].center_y, -Part[obj_n].center_z);
	glTranslatef((float)Part[obj_n].x, (float)Part[obj_n].y, (float)Part[obj_n].z);

	glRotatef((float)Part[obj_n].action_rx, 1.0, 0.0, 0.0);
	glRotatef((float)Part[obj_n].action_ry, 0.0, 1.0, 0.0);
	glRotatef((float)Part[obj_n].action_rz, 0.0, 0.0, 1.0);

	glCallList(Part[obj_n].object);
	glPopMatrix();
	return 0;
}

int gldraw_objects(void) {
	char tmp_str[4096];
	int obj_n = 0;
	strcpy(obj_list, "");
	for (obj_n = 0; obj_n < 100; obj_n++) {
		if (Part[obj_n].view == 1) {
			gldraw_object(obj_n);
			sprintf(tmp_str, "mx my mz rx ry rz %i: %s (View);", obj_n, Part[obj_n].name);
			strcat(obj_list, tmp_str);
		} else if (Part[obj_n].name[0] != 0) {
			sprintf(tmp_str, "mx my mz rx ry rz %i: %s (Hidden);", obj_n, Part[obj_n].name);
			strcat(obj_list, tmp_str);
		}
	}
// 	gui_update_list(WidgetID[OBJECT_LIST], obj_list, 695, 40, 300, SCREEN_Y - 85, 99, "Objects");
	return 0;
}

int glinit_objects(void) {
	int obj_n = 0;
	for (obj_n = 0; obj_n < 100; obj_n++) {
		Part[obj_n].x = 0;
		Part[obj_n].y = 0;
		Part[obj_n].z = -0;
		Part[obj_n].center_x = 0;
		Part[obj_n].center_y = 0;
		Part[obj_n].center_z = 0;
		Part[obj_n].rotate_x = 0;
		Part[obj_n].rotate_y = 0;
		Part[obj_n].rotate_z = 0;
		Part[obj_n].action_rx = 0;
		Part[obj_n].action_ry = 0;
		Part[obj_n].action_rz = 0;
		Part[obj_n].view = 0;
		Part[obj_n].group = 0;
		Part[obj_n].name[0] = 0;
	}
	return 0;
}

int LoadGLTextures(char *filename) {
	SDL_Surface *TextureImage[1]; 
	GLfloat texcoord[4];
	// 16x16x24 Win3.x-BMP
	if (TextureImage[0] = IMG_Load(filename)) {
// 		texture[0] = SDL_GL_LoadTexture(TextureImage[0], texcoord);
		glGenTextures(1, &texture[0] );
		glBindTexture(GL_TEXTURE_2D, texture[0] );
		glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->w, TextureImage[0]->h, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->pixels);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		SDL_FreeSurface(TextureImage[0]);
		return 0;
	}
	return 1;
}

int gl_init(void) {
	int w = 800;
	int h = 600;
/*	printf("GL-Screen BPP: %d\n", SDL_GetVideoSurface()->format->BitsPerPixel);
	printf("\n");
	printf( "Vendor     : %s\n", glGetString( GL_VENDOR ) );
	printf( "Renderer   : %s\n", glGetString( GL_RENDERER ) );
	printf( "Version    : %s\n", glGetString( GL_VERSION ) );
	printf( "Extensions : %s\n", glGetString( GL_EXTENSIONS ) );
	printf("\n");
*/
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glViewport(0, 0, w, h);
	glOrtho(0, 800, 0, 800, -800, 800);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
// 	glShadeModel(GL_FLAT);
	glShadeModel(GL_SMOOTH);

	// Transparenz-Aktivieren
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	static float mat_shininess[] = {1.0};
	static float mat_specular[] = {0.5, 0.4, 0.6, 1.0};
	glMaterialfv (GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
	glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
	
	// Lichtquelle 0 einstellen
	static float light0_ambient[] = {0.7, 0.7, 0.7, 1.0};
	static float light0_diffuse[] = {0.8, 0.8, 0.8, 1.0};
	static float light0_pos[]    = {0.6, 0.6, 0.6, 1.0};
	glLightfv (GL_LIGHT0, GL_AMBIENT, light0_ambient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
	glLightfv (GL_LIGHT0, GL_POSITION,light0_pos);
	// Beleuchtungsmodell waehlen
	static float lmodel_ambient[] = {1.0, 1.0, 1.0, 1.0};
	glLightModelfv (GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glLightModeli (GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
	// ...und Beleuchtung einschalten. 
	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
	glEnable (GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

	return 0;
}

