
#define GUI_MAX_WIDGETS 200
#define MAIN		0
#define GUI_INFO	1
#define GUI_PROGRESS	2


int WidgetID[GUI_MAX_WIDGETS];
char gui_info[256];
int gui_percent = 0;

typedef struct {
	int x;
	int y;
	int w;
	int h;
	int action;
	int view;
	int group;
	int type;
	float zoom;
	char info[64];
	char hotkey[10];
	int (*mdown) (void);
	int (*mover) (void);
	int (*mup) (void);
	SDL_Surface *surface;
} Gui_Element;
Gui_Element Widget[GUI_MAX_WIDGETS];

int (*gui_input) (char *txt_str);
int Widget_NextID = 0;
int gui_running = 1;
// float z_data[1];

int gui_init(void) {
	int wid;
	for (wid = 0; wid < GUI_MAX_WIDGETS; wid++) {
		Widget[wid].x = 0;
		Widget[wid].y = 0;
		Widget[wid].w = 0;
		Widget[wid].h = 0;
		Widget[wid].action = 1;
		Widget[wid].group = 0;
		Widget[wid].view = 0;
		Widget[wid].zoom = 1.0;
		Widget[wid].type = 0;
		Widget[wid].hotkey[0] = 0;
		Widget[wid].surface = NULL;
		Widget[wid].mdown = NULL;
		Widget[wid].mover = NULL;
		Widget[wid].mup = NULL;
		Widget[wid].info[0] = 0;
	}
	Widget_NextID = 0;
	gui_input = NULL;
	gui_running = 1;
	return 0;
}

int gui_exit(void) {
	gui_running = 0;
	SDL_Quit();
	return 0;
}

int gui_update_icon(int wid, char *icon, int start_x, int start_y) {
	char tmp_str[256];
	if (start_x != -1) {
		Widget[wid].x = start_x;
	}
	if (start_y != -1) {
		Widget[wid].y = start_y;
	}
	if (icon[0] != 0) {
		sprintf(tmp_str, "/usr/share/mxm-cnc/icons/%s.png", icon);
		SDL_FreeSurface(Widget[wid].surface);
		Widget[wid].surface = IMG_Load(tmp_str);
		Widget[wid].w = Widget[wid].surface->w;
		Widget[wid].h = Widget[wid].surface->h;
	}
	gui_draw_object(wid);
	return 0;
}

int gui_update_image(int wid, char *image, int start_x, int start_y) {
	char tmp_str[256];
	if (start_x != -1) {
		Widget[wid].x = start_x;
	}
	if (start_y != -1) {
		Widget[wid].y = start_y;
	}
	if (image[0] != 0) {
		sprintf(tmp_str, "%s", image);
		SDL_FreeSurface(Widget[wid].surface);
		Widget[wid].surface = IMG_Load(tmp_str);
		Widget[wid].w = Widget[wid].surface->w;
		Widget[wid].h = Widget[wid].surface->h;
	}
	gui_draw_object(wid);
	return 0;
}

int gui_show_group(int group_id) {
	int wid;
	for (wid = 0; wid < GUI_MAX_WIDGETS; wid++) {
		if (Widget[wid].surface != NULL && Widget[wid].group == group_id) {
			Widget[wid].view = 1;
		}
	}
	return 0;
}

int gui_hide_group(int group_id) {
	int wid;
	for (wid = 0; wid < GUI_MAX_WIDGETS; wid++) {
		if (Widget[wid].surface != NULL && Widget[wid].group == group_id) {
			Widget[wid].view = 0;
		}
	}
	return 0;
}

int gui_select_icon(int wid_selected) {
	int wid;
	for (wid = 0; wid < GUI_MAX_WIDGETS; wid++) {
		if (Widget[wid].surface != NULL && Widget[wid].group == Widget[wid_selected].group) {
			Widget[wid].action = 1;
			gui_draw_object(wid);
		}
	}
 	Widget[wid_selected].action = 2;
 	gui_draw_object(wid_selected);
	return 0;
}

int gui_add_icon(char *icon, int start_x, int start_y, int group, char *gui_info) {
	char tmp_str[256];
	Widget[Widget_NextID].x = start_x;
	Widget[Widget_NextID].y = start_y;
	sprintf(tmp_str, "/usr/share/mxm-cnc/icons/%s.png", icon);
	Widget[Widget_NextID].surface = IMG_Load(tmp_str);
	Widget[Widget_NextID].w = Widget[Widget_NextID].surface->w;
	Widget[Widget_NextID].h = Widget[Widget_NextID].surface->h;
	Widget[Widget_NextID].action = 1;
	Widget[Widget_NextID].group = group;
	Widget[Widget_NextID].zoom = 1.0;
	Widget[Widget_NextID].type = 0;
	Widget[Widget_NextID].view = 1;
	Widget[Widget_NextID].mdown = NULL;
	Widget[Widget_NextID].mover = NULL;
	Widget[Widget_NextID].mup = NULL;
	Widget[Widget_NextID].hotkey[0] = 0;
	strcpy(Widget[Widget_NextID].info, gui_info);
	return Widget_NextID++;
}

int gui_add_image(char *image, int start_x, int start_y, int group, char *gui_info) {
	char tmp_str[256];
	Widget[Widget_NextID].x = start_x;
	Widget[Widget_NextID].y = start_y;
	sprintf(tmp_str, "%s", image);
	Widget[Widget_NextID].surface = IMG_Load(tmp_str);
	Widget[Widget_NextID].w = Widget[Widget_NextID].surface->w;
	Widget[Widget_NextID].h = Widget[Widget_NextID].surface->h;
	Widget[Widget_NextID].action = 1;
	Widget[Widget_NextID].group = group;
	Widget[Widget_NextID].zoom = 1.0;
	Widget[Widget_NextID].hotkey[0] = 0;
	Widget[Widget_NextID].type = 0;
	Widget[Widget_NextID].view = 1;
	Widget[Widget_NextID].mdown = NULL;
	Widget[Widget_NextID].mover = NULL;
	Widget[Widget_NextID].mup = NULL;
	strcpy(Widget[Widget_NextID].info, gui_info);
	return Widget_NextID++;
}

int gui_update_canvas(int wid) {
	return 0;
}

int gui_add_canvas(int start_x, int start_y, int w, int h, int group, char *gui_info) {
	char tmp_str[256];
	Widget[Widget_NextID].x = start_x;
	Widget[Widget_NextID].y = start_y;
	Widget[Widget_NextID].w = w;
	Widget[Widget_NextID].h = h;
	Widget[Widget_NextID].surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, rmask, gmask, bmask, amask);
// 	SDL_SetColorKey(Widget[Widget_NextID].surface, SDL_SRCCOLORKEY, SDL_MapRGB(Widget[Widget_NextID].surface->format, 255, 255, 255));
	Widget[Widget_NextID].action = 0;
	Widget[Widget_NextID].group = group;
	Widget[Widget_NextID].zoom = 1.0;
	Widget[Widget_NextID].hotkey[0] = 0;
	Widget[Widget_NextID].type = 1;
	Widget[Widget_NextID].view = 1;
	Widget[Widget_NextID].mdown = NULL;
	Widget[Widget_NextID].mover = NULL;
	Widget[Widget_NextID].mup = NULL;
	strcpy(Widget[Widget_NextID].info, gui_info);
	return Widget_NextID++;
}

int gui_update_button(int wid, char *text_str) {
	SDL_Rect dstrect;
	SDL_Rect srcrect;
	SDL_Surface *text = NULL;
	SDL_Color color;
	color.r = 0;
	color.g = 0;
	color.b = 0;
	SDL_FreeSurface(Widget[wid].surface);
	Widget[wid].surface = SDL_CreateRGBSurface(SDL_SWSURFACE, Widget[wid].w, Widget[wid].h, 32, rmask, gmask, bmask, amask);
	if ((text = TTF_RenderText_Solid(Stat_Font, text_str, color)) != NULL) {
		srcrect.x = 0;
		srcrect.y = 0;
		srcrect.w = Widget[wid].w;
		srcrect.h = Widget[wid].h;
		dstrect.x = 5;
		dstrect.y = 3;
		dstrect.w = Widget[wid].w;
		dstrect.h = Widget[wid].h;
		SDL_BlitSurface(text, &srcrect, Widget[wid].surface, &dstrect);
		SDL_FreeSurface(text);
	}
	rectangleRGBA(Widget[wid].surface, 0, 0, Widget[wid].surface->w - 1, Widget[wid].surface->h - 1, 0, 0, 0, 255);
	gui_draw_object(wid);
	return 0;
}

int gui_add_button(char *text_str, int start_x, int start_y, int w, int h, int group, char *gui_info) {
	SDL_Rect dstrect;
	SDL_Rect srcrect;
	SDL_Surface *text = NULL;
	SDL_Color color;
	color.r = 0;
	color.g = 0;
	color.b = 0;
	Widget[Widget_NextID].x = start_x;
	Widget[Widget_NextID].y = start_y;
	Widget[Widget_NextID].surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, rmask, gmask, bmask, amask);
	if ((text = TTF_RenderText_Solid(Stat_Font, text_str, color)) != NULL) {
		srcrect.x = 0;
		srcrect.y = 0;
		srcrect.w = w;
		srcrect.h = h;
		dstrect.x = 5;
		dstrect.y = 3;
		dstrect.w = w;
		dstrect.h = h;
		SDL_BlitSurface(text, &srcrect, Widget[Widget_NextID].surface, &dstrect);
		SDL_FreeSurface(text);
	}
	Widget[Widget_NextID].w = w;
	Widget[Widget_NextID].h = h;
	Widget[Widget_NextID].action = 0;
	Widget[Widget_NextID].group = group;
	Widget[Widget_NextID].zoom = 1.0;
	Widget[Widget_NextID].hotkey[0] = 0;
	Widget[Widget_NextID].type = 0;
	Widget[Widget_NextID].view = 1;
	Widget[Widget_NextID].mdown = NULL;
	Widget[Widget_NextID].mover = NULL;
	Widget[Widget_NextID].mup = NULL;
	strcpy(Widget[Widget_NextID].info, gui_info);
	rectangleRGBA(Widget[Widget_NextID].surface, 0, 0, Widget[Widget_NextID].surface->w - 1, Widget[Widget_NextID].surface->h - 1, 0, 0, 0, 255);
	return Widget_NextID++;
}

int gui_update_progress(int wid, int percent) {
	char tmp_str[256];
	SDL_Rect dstrect;
	SDL_Rect srcrect;
	SDL_Surface *text = NULL;
	SDL_Color color;
	color.r = 0;
	color.g = 0;
	color.b = 0;
	SDL_FreeSurface(Widget[wid].surface);
	Widget[wid].surface = SDL_CreateRGBSurface(SDL_SWSURFACE, Widget[wid].w, Widget[wid].h, 32, rmask, gmask, bmask, amask);
	boxRGBA(Widget[wid].surface, 0, 0, (int)((float)(Widget[wid].surface->w) / 100.0 * (float)percent), Widget[wid].surface->h - 1, 0, 255, 0, 255);
	sprintf(tmp_str, "Status: %i%%", percent);
	if ((text = TTF_RenderText_Solid(Stat_Font, tmp_str, color)) != NULL) {
		srcrect.x = 0;
		srcrect.y = 0;
		srcrect.w = Widget[wid].w;
		srcrect.h = Widget[wid].h;
		dstrect.x = 5;
		dstrect.y = 3;
		dstrect.w = Widget[wid].w;
		dstrect.h = Widget[wid].h;
		SDL_BlitSurface(text, &srcrect, Widget[wid].surface, &dstrect);
		SDL_FreeSurface(text);
	}
	rectangleRGBA(Widget[wid].surface, 0, 0, Widget[wid].surface->w - 1, Widget[wid].surface->h - 1, 0, 0, 0, 255);
	gui_draw_object(wid);
	return 0;
}

int gui_add_progress(int percent, int start_x, int start_y, int w, int h, int group, char *gui_info) {
	char tmp_str[256];
	SDL_Rect dstrect;
	SDL_Rect srcrect;
	SDL_Surface *text = NULL;
	SDL_Color color;
	color.r = 0;
	color.g = 0;
	color.b = 0;
	Widget[Widget_NextID].x = start_x;
	Widget[Widget_NextID].y = start_y;
	Widget[Widget_NextID].surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, rmask, gmask, bmask, amask);
	boxRGBA(Widget[Widget_NextID].surface, 0, 0, (int)((float)(Widget[Widget_NextID].surface->w) / 100.0 * (float)percent), Widget[Widget_NextID].surface->h - 1, 0, 255, 0, 255);
	sprintf(tmp_str, "Status: %i%%", percent);
	if ((text = TTF_RenderText_Solid(Stat_Font, tmp_str, color)) != NULL) {
		srcrect.x = 0;
		srcrect.y = 0;
		srcrect.w = w;
		srcrect.h = h;
		dstrect.x = 5;
		dstrect.y = 3;
		dstrect.w = w;
		dstrect.h = h;
		SDL_BlitSurface(text, &srcrect, Widget[Widget_NextID].surface, &dstrect);
		SDL_FreeSurface(text);
	}
	Widget[Widget_NextID].w = w;
	Widget[Widget_NextID].h = h;
	Widget[Widget_NextID].action = 0;
	Widget[Widget_NextID].group = group;
	Widget[Widget_NextID].zoom = 1.0;
	Widget[Widget_NextID].hotkey[0] = 0;
	Widget[Widget_NextID].type = 0;
	Widget[Widget_NextID].view = 1;
	Widget[Widget_NextID].mdown = NULL;
	Widget[Widget_NextID].mover = NULL;
	Widget[Widget_NextID].mup = NULL;
	strcpy(Widget[Widget_NextID].info, gui_info);
	rectangleRGBA(Widget[Widget_NextID].surface, 0, 0, Widget[Widget_NextID].surface->w - 1, Widget[Widget_NextID].surface->h - 1, 0, 0, 0, 255);
	return Widget_NextID++;
}

int gui_update_text(int wid, char *text_str) {
	SDL_Rect dstrect;
	SDL_Rect srcrect;
	SDL_Surface *text = NULL;
	SDL_Color color;
	color.r = 0;
	color.g = 0;
	color.b = 0;
	SDL_FreeSurface(Widget[wid].surface);
	Widget[wid].surface = SDL_CreateRGBSurface(SDL_SWSURFACE, Widget[wid].w, Widget[wid].h, 32, rmask, gmask, bmask, amask);
	if ((text = TTF_RenderText_Solid(Stat_Font, text_str, color)) != NULL) {
		srcrect.x = 0;
		srcrect.y = 0;
		srcrect.w = Widget[wid].w;
		srcrect.h = Widget[wid].h;
		dstrect.x = 5;
		dstrect.y = 3;
		dstrect.w = Widget[wid].w;
		dstrect.h = Widget[wid].h;
		SDL_BlitSurface(text, &srcrect, Widget[wid].surface, &dstrect);
		SDL_FreeSurface(text);
	}
	strcpy(Widget[wid].info, text_str);
	gui_draw_object(wid);
	return 0;
}

int gui_add_text(char *text_str, int start_x, int start_y, int w, int h, int group, char *gui_info) {
	SDL_Rect dstrect;
	SDL_Rect srcrect;
	SDL_Surface *text = NULL;
	SDL_Color color;
	color.r = 0;
	color.g = 0;
	color.b = 0;
	Widget[Widget_NextID].x = start_x;
	Widget[Widget_NextID].y = start_y;
	Widget[Widget_NextID].surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, rmask, gmask, bmask, amask);
	if ((text = TTF_RenderText_Solid(Stat_Font, text_str, color)) != NULL) {
		srcrect.x = 0;
		srcrect.y = 0;
		srcrect.w = w;
		srcrect.h = h;
		dstrect.x = 5;
		dstrect.y = 3;
		dstrect.w = w;
		dstrect.h = h;
		SDL_BlitSurface(text, &srcrect, Widget[Widget_NextID].surface, &dstrect);
		SDL_FreeSurface(text);
	}
	Widget[Widget_NextID].w = w;
	Widget[Widget_NextID].h = h;
	Widget[Widget_NextID].action = 0;
	Widget[Widget_NextID].group = group;
	Widget[Widget_NextID].zoom = 1.0;
	Widget[Widget_NextID].hotkey[0] = 0;
	Widget[Widget_NextID].type = 2;
	Widget[Widget_NextID].view = 1;
	Widget[Widget_NextID].mdown = NULL;
	Widget[Widget_NextID].mover = NULL;
	Widget[Widget_NextID].mup = NULL;
	strcpy(Widget[Widget_NextID].info, gui_info);
	return Widget_NextID++;
}

int gui_update_list(int wid, char *list, int start_x, int start_y, int w, int h, int group, char *gui_info) {
	char tmp_str[4096];
	char *loc1;
	char *loc2;
	int n = 0;
	SDL_Rect dstrect;
	SDL_Rect srcrect;
	SDL_Surface *text = NULL;
	SDL_Color color;
	color.r = 0;
	color.g = 0;
	color.b = 0;
	Widget[wid].x = start_x;
	Widget[wid].y = start_y;
	SDL_FreeSurface(Widget[wid].surface);
	Widget[wid].surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, rmask, gmask, bmask, amask);
	loc2 = list;
	while ((loc1 = strstr(loc2, ";")) != 0) {
		strcpy(tmp_str, loc2);
		tmp_str[strstr(tmp_str, ";") - tmp_str] = 0;
		loc2 = loc1 + 1;
		if ((text = TTF_RenderText_Solid(Stat_Font, tmp_str, color)) != NULL) {
			srcrect.x = 0;
			srcrect.y = 0;
			srcrect.w = w;
			srcrect.h = h;
			dstrect.x = 5;
			dstrect.y = 3 + n++ * 20;
			dstrect.w = w;
			dstrect.h = h;
			SDL_BlitSurface(text, &srcrect, Widget[wid].surface, &dstrect);
			SDL_FreeSurface(text);
		}
	}
	Widget[wid].w = w;
	Widget[wid].h = h;
	Widget[wid].group = group;
	strcpy(Widget[wid].info, gui_info);
	return 0;
}

int gui_add_list(char *list, int start_x, int start_y, int w, int h, int group, char *gui_info) {
	char tmp_str[4096];
	char *loc1;
	char *loc2;
	int n = 0;
	SDL_Rect dstrect;
	SDL_Rect srcrect;
	SDL_Surface *text = NULL;
	SDL_Color color;
	color.r = 0;
	color.g = 0;
	color.b = 0;
	Widget[Widget_NextID].x = start_x;
	Widget[Widget_NextID].y = start_y;
	Widget[Widget_NextID].surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, rmask, gmask, bmask, amask);
	loc2 = list;
	while ((loc1 = strstr(loc2, ";")) != 0) {
		strcpy(tmp_str, loc2);
		tmp_str[strstr(tmp_str, ";") - tmp_str] = 0;
		loc2 = loc1 + 1;
		if ((text = TTF_RenderText_Solid(Stat_Font, tmp_str, color)) != NULL) {
			srcrect.x = 0;
			srcrect.y = 0;
			srcrect.w = w;
			srcrect.h = h;
			dstrect.x = 5;
			dstrect.y = 3 + n++ * 20;
			dstrect.w = w;
			dstrect.h = h;
			SDL_BlitSurface(text, &srcrect, Widget[Widget_NextID].surface, &dstrect);
			SDL_FreeSurface(text);
		}
	}
	Widget[Widget_NextID].w = w;
	Widget[Widget_NextID].h = h;
	Widget[Widget_NextID].action = 0;
	Widget[Widget_NextID].group = group;
	Widget[Widget_NextID].zoom = 1.0;
	Widget[Widget_NextID].hotkey[0] = 0;
	Widget[Widget_NextID].type = 2;
	Widget[Widget_NextID].view = 1;
	Widget[Widget_NextID].mdown = NULL;
	Widget[Widget_NextID].mover = NULL;
	Widget[Widget_NextID].mup = NULL;
	strcpy(Widget[Widget_NextID].info, gui_info);
	return Widget_NextID++;
}

int zoom_rotate(SDL_Surface *Scr, SDL_Surface *Dst, int x, int y, int rotate_val, float zoom_val) {
	SDL_Surface *TMP_Img = NULL;
	SDL_Rect dstrect;
	TMP_Img = rotozoomSurface(Scr, rotate_val, zoom_val, 0);
	dstrect.x = x;
	dstrect.y = y;
	dstrect.w = TMP_Img->w;
	dstrect.h = TMP_Img->h;
	SDL_BlitSurface(TMP_Img, NULL, Dst, &dstrect);
	SDL_FreeSurface(TMP_Img);
	return 0;
}

int gui_draw(void) {
	int wid;
	SDL_Rect dstrect;
	SDL_Rect srcrect;
// 	boxRGBA(WinScreen, 0, 0, SCREEN_X, SCREEN_Y, 255, 255, 255, 255);
	for (wid = 0; wid < GUI_MAX_WIDGETS; wid++) {
		if (Widget[wid].view != 0 && Widget[wid].surface != NULL) {
			gui_draw_object(wid);
		}
	}
	gui_update_button(WidgetID[GUI_INFO], gui_info);
	gui_update_progress(WidgetID[GUI_PROGRESS], gui_percent);
	SDL_GL_SwapBuffers();
	return 0;
}

int gui_draw_object(int wid) {
	SDL_Rect dstrect;
	SDL_Rect srcrect;
	if (Widget[wid].view != 0 && Widget[wid].surface != NULL) {
		srcrect.x = 0;
		srcrect.y = 0;
		srcrect.w = Widget[wid].w;
		srcrect.h = Widget[wid].h;
		dstrect.x = Widget[wid].x;
		dstrect.y = Widget[wid].y;
		dstrect.w = Widget[wid].w;
		dstrect.h = Widget[wid].h;
		boxRGBA(WinScreen, Widget[wid].x, Widget[wid].y, Widget[wid].x + Widget[wid].w, Widget[wid].y + Widget[wid].h, 255, 255, 255, 255);
		if (Widget[wid].zoom != 1.0) {
			zoom_rotate(Widget[wid].surface, WinScreen, Widget[wid].x, Widget[wid].y, 0, Widget[wid].zoom);
		} else {
			SDL_BlitSurface(Widget[wid].surface, &srcrect, WinScreen, &dstrect);
		}
		if (Widget[wid].action == 1) {
 			lineRGBA(WinScreen, Widget[wid].x, Widget[wid].y, Widget[wid].x, Widget[wid].y + Widget[wid].h, 255, 255, 255, 255);
 			lineRGBA(WinScreen, Widget[wid].x, Widget[wid].y, Widget[wid].x + Widget[wid].w, Widget[wid].y, 255, 255, 255, 255);
 			lineRGBA(WinScreen, Widget[wid].x + Widget[wid].w, Widget[wid].y, Widget[wid].x + Widget[wid].w, Widget[wid].y + Widget[wid].h, 0, 0, 0, 255);
 			lineRGBA(WinScreen, Widget[wid].x, Widget[wid].y + Widget[wid].h, Widget[wid].x + Widget[wid].w, Widget[wid].y + Widget[wid].h, 0, 0, 0, 255);
		} else if (Widget[wid].action == 2) {
 			lineRGBA(WinScreen, Widget[wid].x, Widget[wid].y, Widget[wid].x, Widget[wid].y + Widget[wid].h, 0, 0, 0, 255);
 			lineRGBA(WinScreen, Widget[wid].x, Widget[wid].y, Widget[wid].x + Widget[wid].w, Widget[wid].y, 0, 0, 0, 255);
 			lineRGBA(WinScreen, Widget[wid].x + Widget[wid].w, Widget[wid].y, Widget[wid].x + Widget[wid].w, Widget[wid].y + Widget[wid].h, 255, 255, 255, 255);
 			lineRGBA(WinScreen, Widget[wid].x, Widget[wid].y + Widget[wid].h, Widget[wid].x + Widget[wid].w, Widget[wid].y + Widget[wid].h, 255, 255, 255, 255);
		}
		SDL_UpdateRect(WinScreen, Widget[wid].x, Widget[wid].y, Widget[wid].w, Widget[wid].h);
	}
	return 0;
}

int gui_check(void) {
	char tmp_str[512];
	int n = 0;
	int wid;
	while(SDL_PollEvent(&event)) {
		if (event.type == SDL_QUIT) {
			event.type = 0;
			gui_running = 0;
			return 0;
		} else if (event.type == SDL_KEYDOWN) {
			event.type = 0;
			for (wid = 0; wid < GUI_MAX_WIDGETS; wid++) {
				if ((Widget[wid].view == 1 || Widget[wid].type == 4) && Widget[wid].surface != NULL) {
					if (Widget[wid].mdown != NULL && (event.key.keysym.mod & KMOD_CTRL)) {
						if (Widget[wid].hotkey[0] != 0 && strcmp(SDL_GetKeyName(event.key.keysym.sym), Widget[wid].hotkey) == 0) {
							if (Widget[wid].type == 0) {
								Widget[wid].action = 2;
							}
							gui_draw();
							(*Widget[wid].mdown)();
							if (Widget[wid].type == 0) {
								Widget[wid].action = 1;
							}
							return 0;
						}
					}
				}
			}
			if (gui_input != NULL) {
				if (((*gui_input)(SDL_GetKeyName(event.key.keysym.sym))) == 0) {
					return 0;
				}
			}
			if (strcmp(SDL_GetKeyName(event.key.keysym.sym), "escape") == 0) {
				gui_running = 0;
				return 0;
			} else if (strcmp(SDL_GetKeyName(event.key.keysym.sym), "backspace") == 0) {
			} else {
				fprintf(stderr, "Key: %s\n", SDL_GetKeyName(event.key.keysym.sym));
			}
		} else if (event.type == SDL_MOUSEBUTTONDOWN) {
			event.type = 0;
			for (wid = 199; wid >= 0; wid--) {
				if ((Widget[wid].view == 1 || Widget[wid].type == 4) && Widget[wid].surface != NULL) {
					if (Widget[wid].x <= event.button.x && Widget[wid].x + Widget[wid].w >= event.button.x) {
						if (Widget[wid].y <= event.button.y && Widget[wid].y + Widget[wid].h >= event.button.y) {
							if (Widget[wid].mdown != NULL) {
								if (Widget[wid].type == 0) {
									Widget[wid].action = 2;
								}
								gui_draw_object(wid);
								(*Widget[wid].mdown)();
								if (Widget[wid].type == 0) {
									Widget[wid].action = 1;
								}
							}
							return 0;
						}
					}
				}
			}
		} else if (event.type == SDL_MOUSEBUTTONUP) {
			event.type = 0;
			for (wid = 199; wid >= 0; wid--) {
				if ((Widget[wid].view == 1 || Widget[wid].type == 4) && Widget[wid].surface != NULL) {
					if (Widget[wid].x <= event.button.x && Widget[wid].x + Widget[wid].w >= event.button.x) {
						if (Widget[wid].y <= event.button.y && Widget[wid].y + Widget[wid].h >= event.button.y) {
							if (Widget[wid].mup != NULL) {
								if (Widget[wid].type == 0) {
									Widget[wid].action = 2;
								}
								gui_draw_object(wid);
								(*Widget[wid].mup)();
								if (Widget[wid].type == 0) {
									Widget[wid].action = 1;
								}
							}
							return 0;
						}
					}
				}
			}
		} else if (event.type == SDL_MOUSEMOTION) {
			event.type = 0;
			for (wid = 199; wid >= 0; wid--) {
				if ((Widget[wid].view == 1 || Widget[wid].type == 4) && Widget[wid].surface != NULL) {
					if (Widget[wid].x <= event.button.x && Widget[wid].x + Widget[wid].w >= event.button.x) {
						if (Widget[wid].y <= event.button.y && Widget[wid].y + Widget[wid].h >= event.button.y) {
							if (Widget[wid].mover != NULL) {
								(*Widget[wid].mover)();
							} else if (Widget[wid].info[0] != 0) {
								strcpy(gui_info, Widget[wid].info);
								if (Widget[wid].hotkey[0] != 0) {
									strcat(gui_info, " (Ctrl+");
									strcat(gui_info, Widget[wid].hotkey);
									strcat(gui_info, ")");
								}
							}
							break;
						}
					}
				}
			}
		}
	}
	return 0;
}

