
#include <time.h>
#include <SDL.h>
#include <SDL_thread.h>
#include <SDL_ttf.h>
#include <SDL_events.h>
#include <SDL_image.h>
#include <SDL_gfxPrimitives.h>
#include <SDL_rotozoom.h>
#include <SDL_opengl.h>

#define SCREEN_X 900
#define SCREEN_Y 600

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
Uint32 rmask = 0xff000000;
Uint32 gmask = 0x00ff0000;
Uint32 bmask = 0x0000ff00;
Uint32 amask = 0x000000ff;
#else
Uint32 rmask = 0x000000ff;
Uint32 gmask = 0x0000ff00;
Uint32 bmask = 0x00ff0000;
Uint32 amask = 0xff000000;
#endif

SDL_Surface *WinScreen = NULL;
TTF_Font *Stat_Font;
SDL_Event event;

int sdl_init(void) {
	SDL_Rect text_dstrect;
	SDL_Surface *text = NULL;
	SDL_Color color;
	color.r = 0;
	color.g = 0;
	color.b = 0;
	Uint32 flags;
	int i;
	int bpp = 0;
	int rgb_size[3];
	int w = SCREEN_X;
	int h = SCREEN_Y;
	int done = 0;
	int frames;
	Uint32 start_time, this_time;
	Uint32 video_flags;
	int value;
	if( SDL_Init(SDL_INIT_VIDEO | SDL_INIT_EVENTTHREAD | SDL_INIT_TIMER) < 0 ) {
		fprintf(stderr,"Couldn't initialize SDL: %s\n",SDL_GetError());
		exit( 1 );
	}
	if ( bpp == 0 ) {
		if ( SDL_GetVideoInfo()->vfmt->BitsPerPixel <= 8 ) {
			bpp = 8;
		} else {
			bpp = 16;
		}
	}
	switch (bpp) {
	    case 8:
		rgb_size[0] = 3;
		rgb_size[1] = 3;
		rgb_size[2] = 2;
		break;
	    case 15:
	    case 16:
		rgb_size[0] = 5;
		rgb_size[1] = 5;
		rgb_size[2] = 5;
		break;
            default:
		rgb_size[0] = 8;
		rgb_size[1] = 8;
		rgb_size[2] = 8;
		break;
	}
	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, rgb_size[0] );
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, rgb_size[1] );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, rgb_size[2] );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	video_flags = SDL_OPENGLBLIT | SDL_RESIZABLE;
	if ((WinScreen = SDL_SetVideoMode(w, h, bpp, video_flags)) == NULL) {
		fprintf(stderr, "Couldn't set GL mode: %s\n", SDL_GetError());
		SDL_Quit();
		exit(1);
	}
	if (TTF_Init() < 0) {
		fprintf(stderr, "Couldn't initialize TTF: %s\n", SDL_GetError());
		return -1;
	}
	Stat_Font = TTF_OpenFont("/usr/share/fonts/truetype/ttf-bitstream-vera/Vera.ttf", 12);

	gl_init();

	return 0;
}

int sdl_print_keyinfo(SDL_KeyboardEvent *key) {
	return key->keysym.scancode;
}

char *sdl_printkeyname(SDL_KeyboardEvent *key) {
	return(SDL_GetKeyName(key->keysym.sym));
}

int sdl_rotate(SDL_Surface *Scr, SDL_Surface *Dst, int x, int y, int rotate_val) {
	SDL_Surface *TMP_Img = NULL;
	SDL_Rect dstrect;
	TMP_Img = rotozoomSurface(Scr, rotate_val, 1, 0);
	dstrect.x = x;
	dstrect.y = y;
	dstrect.w = TMP_Img->w;
	dstrect.h = TMP_Img->h;
	SDL_BlitSurface(TMP_Img, NULL, Dst, &dstrect);
	SDL_FreeSurface(TMP_Img);
	return 0;
}

Uint32 sdl_getpixel(SDL_Surface *surface, int x, int y) {
	int bpp = surface->format->BytesPerPixel;
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
	switch(bpp) {
		case 1: {
			return *p;
		}
		case 2: {
			return *(Uint16 *)p;
		}
		case 3: {
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
			return p[0] << 16 | p[1] << 8 | p[2];
			else
			return p[0] | p[1] << 8 | p[2] << 16;
		}
		case 4: {
			return *(Uint32 *)p;
		}
		default: {
			return 0;
		}
	}
}

Uint32 sdl_getpixel_fast(SDL_Surface *surface, int x, int y) {
	return *(Uint32 *)((Uint8 *)surface->pixels + y * surface->pitch + x * surface->format->BytesPerPixel);
}



